(setq debug-on-error t)

;;; Minibuffer frame: ********************************
;;;
(defvar 1on1-minibuffer-frame nil "")

(defcustom 1on1-minibuffer-frame-alist
  (list
   (assq 'foreground-color minibuffer-frame-alist)
   (or (assq 'background-color minibuffer-frame-alist)
       (cons 'background-color "LightBlue"))
   (assq 'font minibuffer-frame-alist)
   (assq 'mouse-color minibuffer-frame-alist)
   (assq 'cursor-color minibuffer-frame-alist)
   (or (assq 'menu-bar-lines minibuffer-frame-alist)
       (cons 'menu-bar-lines 0))
   (or (assq 'left minibuffer-frame-alist)
       (cons 'left 0))
   (or (assq 'height minibuffer-frame-alist)
       (cons 'height 2))
   (or (assq 'icon-type minibuffer-frame-alist)
       (cons 'icon-type (< emacs-major-version 21))) ; `t' for Emacs 21 too?
   (or (assq 'minibuffer minibuffer-frame-alist)
       (cons 'minibuffer 'only))
   (or (assq 'user-position minibuffer-frame-alist)
       (cons 'user-position t))
   (or (assq 'vertical-scroll-bars minibuffer-frame-alist) ;  No scroll bar.
       (cons 'vertical-scroll-bars nil))
   (or (assq 'name minibuffer-frame-alist)
       (cons 'name "Emacs Minibuffer")))
  ""
  :type '(repeat (cons :format "%v" (symbol :tag "Frame Parameter") (sexp :tag "Value")))
  :group 'One-On-One)

;;; *Completions* frame: ********************************
(defcustom 1on1-default-frame-alist
  (list
   (assq 'foreground-color default-frame-alist)
   (or (assq 'background-color default-frame-alist)
       (cons 'background-color "LightBlue"))
   (assq 'font default-frame-alist)
   (assq 'mouse-color default-frame-alist)
   (assq 'cursor-color default-frame-alist)
   (assq 'cursor-type default-frame-alist)
   (or (assq 'menu-bar-lines default-frame-alist)
       (cons 'menu-bar-lines 1))
   (or (assq 'top default-frame-alist)
       (cons 'top 0))
   (or (assq 'left default-frame-alist)
       (cons 'left 0))
   (or (assq 'width default-frame-alist)
       (cons 'width 80))
   (or (assq 'height default-frame-alist)
       (cons 'height 35))
   (or (assq 'minibuffer default-frame-alist)
       (cons 'minibuffer nil))
   (or (assq 'user-position default-frame-alist)
       (cons 'user-position t))
   (or (assq 'vertical-scroll-bars default-frame-alist)
       (cons 'vertical-scroll-bars 'right))
   (or (assq 'icon-type default-frame-alist)
       (cons 'icon-type (< emacs-major-version 21))) ; `t' for Emacs 21 too?
   (or (assq 'tool-bar-lines default-frame-alist)
       (cons 'tool-bar-lines 1))        ; Emacs 21+
   (if (cdr (assq 'left-fringe default-frame-alist))
       (assq 'left-fringe default-frame-alist)
     (cons 'left-fringe 0))             ; Emacs 21+
   (if (cdr (assq 'right-fringe default-frame-alist))
       (assq 'right-fringe default-frame-alist)
     (cons 'right-fringe 0))            ; Emacs 21+
   (or (assq 'fringe default-frame-alist)
       (cons 'fringe 0)))               ; Emacs 21, but not 21.3.50 - REMOVE after 22.x
  ""
  :type '(repeat (cons :format "%v" (symbol :tag "Frame Parameter") (sexp :tag "Value")))
  :group 'frames)

;;; Special-display frames
(defcustom 1on1-special-display-frame-alist
  (list
   (assq 'font special-display-frame-alist)
   (or (assq 'width special-display-frame-alist)
       (cons 'width 80))
   (or (assq 'height special-display-frame-alist)
       (cons 'height 20))
   (assq 'mouse-color special-display-frame-alist)
   (assq 'cursor-color special-display-frame-alist)
   (or (assq 'menu-bar-lines special-display-frame-alist)
       (cons 'menu-bar-lines 1))
   (assq 'foreground-color special-display-frame-alist)
   (or (assq 'background-color special-display-frame-alist)
       (cons 'background-color "LightSteelBlue"))
   (or (assq 'top special-display-frame-alist)
       (cons 'top 0))
   (or (assq 'left special-display-frame-alist)
       (cons 'left 0))
   (or (assq 'unsplittable special-display-frame-alist)
       (cons 'unsplittable t))
   (or (assq 'user-position special-display-frame-alist)
       (cons 'user-position t))
   (or (assq 'vertical-scroll-bars special-display-frame-alist)
       (cons 'vertical-scroll-bars 'right)))
  ""
  :type '(repeat (cons :format "%v" (symbol :tag "Frame Parameter") (sexp :tag "Value")))
  :group 'frames)

(defun 1on1-emacs ()
  ""
  (interactive)
  (unless (if (fboundp 'display-graphic-p) (display-graphic-p) window-system)
    (error "Use `1on1-emacs' only with a graphics display, not with a text terminal"))
  (setq default-frame-alist (append 1on1-default-frame-alist default-frame-alist)
        special-display-frame-alist (append 1on1-special-display-frame-alist
                                            special-display-frame-alist))
  (add-to-list                          ; *Help* frame
   'special-display-buffer-names
   (list "*Help*" '1on1-display-*Help*-frame
         (list (cons 'background-color "Thistle")
               '(height . 40))))
  (add-to-list                          ; *Completions* frame
   'special-display-buffer-names
   `("*Completions*" 1on1-display-*Completions*-frame
     ((background-color . "LavenderBlush2")
      (menu-bar-lines . 0) (tool-bar-lines . 0)
      (width . 100))))
  (setq pop-up-frames  t)
  ;; Minibuffer frame
  (setq minibuffer-frame-alist (append 1on1-minibuffer-frame-alist
                                       minibuffer-frame-alist))

  (if 1on1-minibuffer-frame
      (modify-frame-parameters 1on1-minibuffer-frame 1on1-minibuffer-frame-alist)
    (setq 1on1-minibuffer-frame
          (let ((after-make-frame-functions  nil)) ; E.g. inhibit `fit-frame'.
            (make-frame 1on1-minibuffer-frame-alist))))
;;   (if 1on1-minibuffer-frame
;;       (modify-frame-parameters 1on1-minibuffer-frame 1on1-minibuffer-frame-alist)
;;     (setq 1on1-minibuffer-frame
;;           (make-frame 1on1-minibuffer-frame-alist)))
  (set-frame-width 1on1-minibuffer-frame 150)
  (modify-frame-parameters 1on1-minibuffer-frame `((top ,@ -50)))
  (setq minibuffer-auto-raise t)

  ;; Hooks.
  ;; @@@
  (add-hook 'post-command-hook '1on1-fit-minibuffer-frame)
  (setq w32-grab-focus-on-raise    nil)
  ;; @@@
  )
     
(defun 1on1-display-*Help*-frame (buf &optional args)
  ""
  (let (return-window)
    (setq return-window (select-window (funcall special-display-function buf args)))
    (raise-frame)
    return-window))

(defun 1on1-display-*Completions*-frame (buf &optional args)
  ""
  (let (return-window)
    (setq return-window (select-window (funcall special-display-function buf args)))
    (raise-frame)
    (when (boundp '1on1-minibuffer-frame)
      (redirect-frame-focus (selected-frame) 1on1-minibuffer-frame))
    return-window))

;; @@@
(defun 1on1-fit-minibuffer-frame ()
  ""
  (interactive)
  (when (and (active-minibuffer-window)
             (save-selected-window
               (select-window (minibuffer-window))
               (one-window-p nil 'selected-frame)))
    (let* ((frame         (save-selected-window
                            (select-window (minibuffer-window)) (selected-frame)))
           (frame-height  (frame-height frame)))
      (cond
        ((eq last-command '1on1-fit-minibuffer-frame)
         (set-frame-height frame (1+ (frame-height frame)))
         (modify-frame-parameters 1on1-minibuffer-frame `((top ,@ -50)))
         (condition-case nil (scroll-down (frame-height frame)) (error nil)))
        (t
         (let ((window-min-height  2))
           (modify-frame-parameters 1on1-minibuffer-frame `((top ,@ -50)))
           (condition-case nil (scroll-down (frame-height frame)) (error nil))))))))

;; @@@


(load-file "c:/drews-lisp-20/window-2011-06-23a-MARTIN.el")
